-------------------------------------------------------------------------------
-- File name : maincontrol.vhdl
-- Title     : Main control 
-- project   : ECE 485
-- Library   : single-cycle processor
-- Author(s) : James E. Stine, Jr.
-- Purpose   : definition of single-cycle processor
-- notes     :

-- Copyright Illinois Institute of Technology
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity maincontrol is
  port (Opcode   : in  std_logic_vector(5 downto 0);
        RegDst   : out std_logic;
        ALUSrc   : out std_logic;
        MemtoReg : out std_logic;
        RegWrite : out std_logic;
        MemRead  : out std_logic;
        MemWrite : out std_logic;
        Branch   : out std_logic;
        ALUop    : out std_logic_vector(1 downto 0));
end maincontrol;

architecture structure of MainControl is
begin

  process (Opcode)
  begin
    case Opcode(5 downto 0) is
      when "000000" =>regdst   <= '1';
                      alusrc   <= '0';
                      memtoreg <= '0';
                      regwrite <= '1';
                      memread  <= '0';
                      memwrite <= '0';
                      branch   <= '0';
                      aluop    <= "10";
      when "100011" =>regdst   <= '0';
                      alusrc   <= '1';
                      memtoreg <= '1';
                      regwrite <= '1';
                      memread  <= '1';
                      memwrite <= '0';
                      branch   <= '0';
                      aluop    <= "00";
      when "101011" =>regdst   <= '-';
                      alusrc   <= '1';
                      memtoreg <= '-';
                      regwrite <= '0';
                      memread  <= '0';
                      memwrite <= '1';
                      branch   <= '0';
                      aluop    <= "00";
      when "000100" =>regdst   <= '-';
                      alusrc   <= '0';
                      memtoreg <= '-';
                      regwrite <= '0';
                      memread  <= '0';
                      memwrite <= '0';
                      branch   <= '1';
                      aluop    <= "01";
      when others  => regdst   <= '-';
                      alusrc   <= '-';
                      memtoreg <= '-';
                      regwrite <= '-';
                      memread  <= '-';
                      memwrite <= '-';
                      branch   <= '-';
                      aluop    <= "--";
    end case;
  end process;

end structure;
