-------------------------------------------------------------------------------
-- File name : registerfile.vhdl
-- Title     : 32 x 32-bit register file
-- project   : ECE 485
-- Library   : single-cycle processor
-- Author(s) : James E. Stine, Jr.
-- Purpose   : definition of single-cycle processor
-- notes :   

-- Copyright Illinois Institute of Technology
-------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;

entity registerfile is
  port (ReadReg1  : in  std_logic_vector(4 downto 0);
        ReadReg2  : in  std_logic_vector(4 downto 0);
        WriteReg  : in  std_logic_vector(4 downto 0);
        WriteData : in  std_logic_vector(31 downto 0);
        RegWrite  : in  std_logic;
        clk       : in  std_logic;
        ReadData1 : out std_logic_vector(31 downto 0);
        ReadData2 : out std_logic_vector(31 downto 0));
end registerfile;


architecture structure of registerfile is

  component register32
    port (A   : in  std_logic_vector(31 downto 0);
          clk : in  std_logic;
          Z   : out std_logic_vector(31 downto 0));
  end component;

  component decode5x32
    port (Sel : in  std_logic_vector(4 downto 0);
          Clk : in  std_logic;
          Q   : out std_logic_vector(31 downto 0));
  end component;

  component mux321x32 
    port (D0  : in  std_logic_vector(31 downto 0);
          D1  : in  std_logic_vector(31 downto 0);
          D2  : in  std_logic_vector(31 downto 0);
          D3  : in  std_logic_vector(31 downto 0);
          D4  : in  std_logic_vector(31 downto 0);
          D5  : in  std_logic_vector(31 downto 0);
          D6  : in  std_logic_vector(31 downto 0);
          D7  : in  std_logic_vector(31 downto 0);
          D8  : in  std_logic_vector(31 downto 0);
          D9  : in  std_logic_vector(31 downto 0);
          D10 : in  std_logic_vector(31 downto 0);
          D11 : in  std_logic_vector(31 downto 0);
          D12 : in  std_logic_vector(31 downto 0);
          D13 : in  std_logic_vector(31 downto 0);
          D14 : in  std_logic_vector(31 downto 0);
          D15 : in  std_logic_vector(31 downto 0);
          D16 : in  std_logic_vector(31 downto 0);
          D17 : in  std_logic_vector(31 downto 0);
          D18 : in  std_logic_vector(31 downto 0);
          D19 : in  std_logic_vector(31 downto 0);
          D20 : in  std_logic_vector(31 downto 0);
          D21 : in  std_logic_vector(31 downto 0);
          D22 : in  std_logic_vector(31 downto 0);
          D23 : in  std_logic_vector(31 downto 0);
          D24 : in  std_logic_vector(31 downto 0);
          D25 : in  std_logic_vector(31 downto 0);
          D26 : in  std_logic_vector(31 downto 0);
          D27 : in  std_logic_vector(31 downto 0);
          D28 : in  std_logic_vector(31 downto 0);
          D29 : in  std_logic_vector(31 downto 0);
          D30 : in  std_logic_vector(31 downto 0);
          D31 : in  std_logic_vector(31 downto 0);
          S   : in  std_logic_vector(4 downto 0);
          Y   : out std_logic_vector(31 downto 0));
  end component;

  signal WriteOut : std_logic_vector(31 downto 0);
  signal Q0, Q1, Q2, Q3, Q4 : std_logic_vector(31 downto 0);
  signal Q5, Q6, Q7, Q8, Q9 : std_logic_vector(31 downto 0);
  signal Q10, Q11, Q12, Q13, Q14 : std_logic_vector(31 downto 0);
  signal Q15, Q16, Q17, Q18, Q19 : std_logic_vector(31 downto 0);
  signal Q20, Q21, Q22, Q23, Q24 : std_logic_vector(31 downto 0);
  signal Q25, Q26, Q27, Q28, Q29 : std_logic_vector(31 downto 0);
  signal Q30, Q31 : std_logic_vector(31 downto 0);  
 
begin

  dec1  : decode5x32 port map (WriteReg, clk, WriteOut);

  Reg0  : register32 port map(WriteData, WriteOut(0), Q0);
  Reg1  : register32 port map(WriteData, WriteOut(1), Q1);
  Reg2  : register32 port map(WriteData, WriteOut(2), Q2);
  Reg3  : register32 port map(WriteData, WriteOut(3), Q3);
  Reg4  : register32 port map(WriteData, WriteOut(4), Q4);
  Reg5  : register32 port map(WriteData, WriteOut(5), Q5);
  Reg6  : register32 port map(WriteData, WriteOut(6), Q6);
  Reg7  : register32 port map(WriteData, WriteOut(7), Q7);
  Reg8  : register32 port map(WriteData, WriteOut(8), Q8);
  Reg9  : register32 port map(WriteData, WriteOut(9), Q9);
  Reg10 : register32 port map(WriteData, WriteOut(10), Q10);
  Reg11 : register32 port map(WriteData, WriteOut(11), Q11);
  Reg12 : register32 port map(WriteData, WriteOut(12), Q12);
  Reg13 : register32 port map(WriteData, WriteOut(13), Q13);
  Reg14 : register32 port map(WriteData, WriteOut(14), Q14);
  Reg15 : register32 port map(WriteData, WriteOut(15), Q15);
  Reg16 : register32 port map(WriteData, WriteOut(16), Q16);
  Reg17 : register32 port map(WriteData, WriteOut(17), Q17);
  Reg18 : register32 port map(WriteData, WriteOut(18), Q18);
  Reg19 : register32 port map(WriteData, WriteOut(19), Q19);
  Reg20 : register32 port map(WriteData, WriteOut(20), Q20);
  Reg21 : register32 port map(WriteData, WriteOut(21), Q21);
  Reg22 : register32 port map(WriteData, WriteOut(22), Q22);
  Reg23 : register32 port map(WriteData, WriteOut(23), Q23);
  Reg24 : register32 port map(WriteData, WriteOut(24), Q24);
  Reg25 : register32 port map(WriteData, WriteOut(25), Q25);
  Reg26 : register32 port map(WriteData, WriteOut(26), Q26);
  Reg27 : register32 port map(WriteData, WriteOut(27), Q27);
  Reg28 : register32 port map(WriteData, WriteOut(28), Q28);
  Reg29 : register32 port map(WriteData, WriteOut(29), Q29);
  Reg30 : register32 port map(WriteData, WriteOut(30), Q30);
  Reg31 : register32 port map(WriteData, WriteOut(31), Q31);

  mux1 :   mux321x32 port map (Q0, Q1, Q2, Q3, Q4, Q5, Q6, Q7, Q8, Q9,
                              Q10, Q11, Q12, Q13, Q14, Q15, Q16, Q17, Q18, Q19,
                              Q20, Q21, Q22, Q23, Q24, Q25, Q26, Q27, Q28, Q29,
                              Q30, Q31, ReadReg1, ReadData1);

  mux2 :   mux321x32 port map (Q0, Q1, Q2, Q3, Q4, Q5, Q6, Q7, Q8, Q9,
                               Q10, Q11, Q12, Q13, Q14, Q15, Q16, Q17, Q18, Q19,
                               Q20, Q21, Q22, Q23, Q24, Q25, Q26, Q27, Q28, Q29,
                               Q30, Q31, ReadReg2, ReadData2);
  
end structure;
